import utf8 from 'utf8';

const PREFIX = {
  SENSOR: 'sen',
  VARIABLE: 'var'
}

/**
 * TOIMPROVE: 更好的做法是做成一个工厂模式，已适配不同的缓存
 */
class CachePool {
  constructor() {
    this.cache_ = {};
  }

  setValue_(id, val) {
    this.cache_[id] = val;
  }

  getValue_(id) {
    return this.cache_[id];
  }

  clear_(type) {
    if (typeof type === 'undefined') {
      this.cache_ = {};
    } else {
      for (let key in this.cache_) {
        if (key.startsWith(type)) {
          delete this.cache_[key];
        }
      }
    }
  }

  /*** Sensor ***/
  /**
   * 设置传感器值
   * @param {String} id  [description]
   * @param {String|Number|Boolean} val [description]
   */
  setSensorValue(id, val) {
    let newid = PREFIX.SENSOR + '_' + id;
    this.setValue_(newid, val);
  }

  getSensorValue(id) {
    let newid = PREFIX.SENSOR + '_' + id;
    return this.getValue_(newid);
  }

  /**
   * 清空传感器缓存
   */
  clearSensorCache() {
    this.clear_(PREFIX.SENSOR);
  }

  /*** CommVar ***/
  /**
   * 设置通信变量值
   * @param {String} id  [description]
   * @param {String|Number|Boolean} val [description]
   */
  setCommVarValue(id, val) {
    let newid = PREFIX.VARIABLE + '_' + id;
    this.setValue_(newid, val);
  }

  getCommVarValue(id) {
    let newid = PREFIX.VARIABLE + '_' + id;
    let value = this.getValue_(newid);
    if (typeof value === 'string') {
      value = value && utf8.decode(value);
    }
    return value;
  }

  /**
   * 清空通信变量缓存
   */
  clearCommVarCache() {
    this.clear_(PREFIX.VARIABLE);
  }

  /*** all cache ***/
  clearAllCache() {
    this.clear_();
  }
}

export default new CachePool();