import {
    ascii2Str,
} from './util';
import Logger from './logger';
import {
    REQUEST_TOPIC,
    POTOCOL_ID,
    MAC_TYPE,
    READ_ACTION,
    EVENT,
    READY_CMD,
    TIPS,
    CORRECTION_MODE,
    DO_CORRECT

} from './setting';

import Decodebytes from './decodebytes';
import Parse from './parse';
import CachePool from './cachepool';

/**
 * 负责返回协议的解析
 */

const Dispatch = {
    emiter: null,
    dispatcherList: [],
    init(emiter) {
        this.emiter = emiter;
    },
    /**
     * [doDispatch description]
     * @param  {Function} match [description]
     * @param  {Function} dispatch [description]
     * @return {[type]}        [description]
     */
    addDispatcher({match=()=>[], dispatch=()=>{}}) {
        this.dispatcherList.push(new dispatchCreater(match, dispatch));

    },
    /**
     * [doDispatch description]
     * @param  {[type]} result [description]
     * @return {[type]}        [description]
     */
    doDispatch(buffData) {
        let frames = Decodebytes.splitFrame(buffData); // 分出帧
        if (frames) { // 若干个完整帧
            // 走分发管道
            for (let dispatcher of this.dispatcherList) {
                try{
                    dispatcher.match(frames).dispatch();
                }catch(e) {
                    console.error(e);
                }
            }
        }
        // log 信息
        else if (Logger.getState()) {
            let data = Parse.getlogInfo(buffData);
            if (data.length) {
                console.log(ascii2Str(data) || '');
            }
        }
    }
}

class dispatchCreater {
    constructor(match, dispatch) {
        this.temp_ = [];
        this.match = function (buffData) {
            this.temp_ = match(buffData);
            return this;
        }
        this.dispatch = function () {
            if(this.temp_.length){
                dispatch(this.temp_);
                this.temp_ = [];
            }
            return this;
        }
    }
}

// 文件协议分发器
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.FILE),
    dispatch: frames => Dispatch.emiter.emit(REQUEST_TOPIC.FLASH_FILE, frames[0])
})

// comm 通信变量协议分发器
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.COMM_VAR),
    dispatch: frames => frames.forEach(val => {
        let res = Parse.getNameAndValue(val);
        CachePool.setCommVarValue(res.name, res.value);
    })
})

// sensor读值协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.SENSOR),
    dispatch: frames => frames.forEach(val => {
        let res = Parse.getSensorVal(val);
        if (res.action === READ_ACTION.VALUE) {// 成功后的值
            CachePool.setSensorValue(res.id, res.value);
        }
        else { // 成功消息 or 失败消息
            let isSuccess = res.action === READ_ACTION.SUCCESS ? true : false;
            Dispatch.emiter.emit(REQUEST_TOPIC.READ_SENSOR, res, isSuccess);
        }
    })
})

// 广播协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.BROADCAST),
    dispatch: frames => frames.forEach(val => {
        //考虑下位机快速返回值，可能一次发送多个帧，但每个帧携带的信息量都不同
        let msg = Parse.getBroadcastMsg(val);
        Dispatch.emiter.emit(EVENT.BROADCAST, msg);
    })
})

// mac 地址协议
// 使用时多半会wifi ble一起查询，会产生粘包。故要用forEach循环处理
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.MAC_ADDR),
    dispatch: frames => frames.forEach(val => {
        let res = Parse.getMacAddr(val);
        if (res.type == MAC_TYPE['1']) {
            Dispatch.emiter.emit(REQUEST_TOPIC.WIFI_MAC, res);
        } else if (res.type == MAC_TYPE['2']) {
            Dispatch.emiter.emit(REQUEST_TOPIC.BLE_MAC, res);
        }
    })
})

// version 查询协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.VERSION),
    dispatch: frames => {
        let res = Parse.getVersion(frames[0]);
        Dispatch.emiter.emit(REQUEST_TOPIC.VERSION, res);
    }
})

// HANDSHAKE_DTR 协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.HANDSHAKE_DTR),
    dispatch: frames => {
        let res = Parse.getHandshakeMsg(frames[0]);
        Dispatch.emiter.emit(REQUEST_TOPIC.HANDSHAKE_DTR, res);
    }
})

// ready 协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.READY),
    dispatch: frames => {
        let res = Parse.getReadyResult(frames[0]);
        if (res.type === READY_CMD.REQUEST_RESULT) { // 主动查询
            Dispatch.emiter.emit(REQUEST_TOPIC.READY_STATUS, {
                tips: TIPS.solt_request,
                status: TIPS.ready
            });
        } else if (res.type === READY_CMD.MACHINE_PUSH) {// 下位机推送
            Dispatch.emiter.emit(EVENT.READY, {
                tips: TIPS.machine_push,
                status: TIPS.ready
            });
        }
    }
})

// 通信 channel 协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.COMM_CHANNEL),
    dispatch: frames => {
        let res = Parse.getCommChannel(frames[0]);
        Dispatch.emiter.emit(REQUEST_TOPIC.COMM_CHANNEL, res);
    }
})

// workmode 协议
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.WORKING_MODE),
    dispatch: frames => {
        let res = Parse.getWorkingMode(frames[0]);
        Dispatch.emiter.emit(REQUEST_TOPIC.WORKING_MODE, res);
    }
})

// SENSOR_CORRECTION 协议 1
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.SENSOR_CORRECTION && val[5] === CORRECTION_MODE.SUB_ID),
    dispatch: frames => {
        let res = Parse.getSensorCorrection(frames[0]);
        let isSuccess = true;
        if (res.result === CORRECTION_MODE.RESULT.SUCC) {
            res.msg = CORRECTION_MODE.MSG.succ; // 成功消息
        } else {
            isSuccess = false;
            res.msg = CORRECTION_MODE.MSG.fail; // 失败消息
        }
        if (res.mode === CORRECTION_MODE.MODE.ENTER) {
            Dispatch.emiter.emit(REQUEST_TOPIC.ENTER_CORRECTION_MODE, res, isSuccess);
        } else if (res.mode === CORRECTION_MODE.MODE.QUIT) {
            Dispatch.emiter.emit(REQUEST_TOPIC.QUIT_CORRECTION_MODE, res, isSuccess);
        }
    }
})

// SENSOR_CORRECTION 协议 2
Dispatch.addDispatcher({
    match: frames => frames.filter((val) => val[4] === POTOCOL_ID.SENSOR_CORRECTION && val[5] === DO_CORRECT.SUB_ID),
    dispatch: frames => {
        let res = Parse.getSensorCorrection(frames[0]);
        let isSuccess = true;
        if (res.result === DO_CORRECT.RESULT.SUCC) {
            res.msg = DO_CORRECT.MSG.succ; // 成功消息
        } else {
            isSuccess = false;
            res.msg = DO_CORRECT.MSG.fail; // 失败消息
        }
        Dispatch.emiter.emit(REQUEST_TOPIC.DO_CORRECT, res, isSuccess);
    }
})

// 未来每增加一条协议，需按以上方式调用 Dispatch.addDispatcher


export default Dispatch;