// 协议帧头
const MEOS_FRAME_HEAD = 0xf3;
// 协议帧尾
const MEOS_FRAME_END = 0xf4;

// 协议族各协议ID
const POTOCOL_ID = {
    'FILE': 0x01,
    'COMM_VAR': 0x02,
    'BROADCAST': 0x03,
    'SENSOR': 0x04,
    'MAC_ADDR': 0x05,
    'VERSION': 0x06,
    'HANDSHAKE_DTR': 0x07,
    'READY': 0x08, // ready 协议
    'COMM_CHANNEL': 0x09,
    'WORKING_MODE': 0X0d,
    'SENSOR_CORRECTION': 0x0e
};

// 请求主题
const REQUEST_TOPIC = {
    'VERSION': Symbol('version'),
    'HANDSHAKE_DTR': Symbol('handshake_dtr'),
    'WIFI_MAC': Symbol('wifi_mac'),
    'BLE_MAC': Symbol('ble_mac'),
    'READ_SENSOR': Symbol('read_sensor'),
    'FLASH_FILE': Symbol('flash_file'),
    'READY_STATUS': Symbol('ready_status'),
    'COMM_CHANNEL': Symbol('comm_channel'),
    'WORKING_MODE': Symbol('working_mode'),
    'ENTER_CORRECTION_MODE': Symbol('enter_correction'),
    'QUIT_CORRECTION_MODE': Symbol('quit_correction'),
    'DO_CORRECT': Symbol('do_correct')
}

// mac 地址类型
const MAC_TYPE = {
    1: 'wifi',
    2: 'ble',
    'WIFI': 1,
    'BLE': 2
};

// 工作模式，在线离线
const WORKING_MODE = {
    0: 'offline',
    1: 'online',
    'offline': 0,
    'online': 1
};

const WORKING_MODE_CMD = {
    'SET': 0x00,
    'GET': 0x80
}

// handshake dtr
const HANDSHAKE_DTR = {
    'REQUEST_DTR': 0x01,
    'RESPONSE_DTR': 0x02
}

// read 协议包含请求和机器主动推送
const READY_CMD = {
    'REQUEST': 0x00,
    'REQUEST_RESULT': 0x80,
    'MACHINE_PUSH': 0xc0
}

// 传感器校正
const CORRECTION_MODE = {
    SUB_ID: 0x01,
    MODE: {
        'ENTER': 0x01,
        'QUIT': 0x02
    },
    RESULT: {
        'SUCC': 0x00,
        'FAIL': 0x01
    },
    MSG: {
        'succ': 'success',
        'fail': 'fail'
    }
}

const DO_CORRECT = {
    SUB_ID: 0x02,
    SENSOR: {
        'GYRO': 0x01,
        'COLOR': 0x02,
        // bellow is sensor name
        1: 'gyro',
        2: 'color'
    },
    RESULT: {
        'SUCC': 0x00,
        'NOT_FOUND': 0x01,
        'NOT_READY': 0x02,
        'OTHER_FAIL': 0x80
    },
    MSG: {
        'succ': 'success',
        'fail': 'fail'
    }
}

// 事件名
const EVENT = {
    'UPLOAD_PROGRESS': Symbol('uploadProgress'),
    'BROADCAST': Symbol('broadcast'),
    'READY': Symbol('ready')
}

// 读值步骤
const READ_ACTION = {
    'READ': 0x01,
    'UNREAD': 0x02,
    'VALUE': 0x03,
    'SUCCESS': 0x04,
    'FAIL': 0x05
}

// 文件上传 cmd id
const FILE_CMD_ID = {
    'HEAD': 0x01,  //发送文件头
    'BODY': 0x02,  //发送文件块
    'DELETE': 0x03,  //删除文件
    'RESPONSE': 0xf0  //响应
}

// 传感器 ID
const SENSOR_ID_MAP = {
    100: {
        type: 'uint32'
    }
}

const TIPS = {
    solt_request: 'result from solt request',
    machine_push: 'result from machine push',
    ready: 'ready',
    unready: 'not ready',
    dtr_ready: 'DTR is ready',
    codey: 'codey'
}

// 默认的信道 index
// 协议涉及兼容问题，该 index 在本 meoser 协议无需关注
const CHANNEL_INDEX = 0x00;


export {
    MEOS_FRAME_HEAD,
    MEOS_FRAME_END,
    POTOCOL_ID,
    FILE_CMD_ID,
    SENSOR_ID_MAP,
    MAC_TYPE,
    HANDSHAKE_DTR,
    READ_ACTION,
    REQUEST_TOPIC,
    READY_CMD,
    EVENT,
    CHANNEL_INDEX,
    TIPS,
    WORKING_MODE,
    WORKING_MODE_CMD,
    CORRECTION_MODE,
    DO_CORRECT
}