import Encodebytes from '../src/encodebytes';
import chai from 'chai';
const expect = chai.expect;

/**
 * 本测试集，主要用来测试协议组装结果
 */

// 通信变量协议
const keyAndValues = [
    {
        desc: `test: 写入 变量名: 'name', 变量值: 100`,
        data: ['name', 100],
        expect: [ 243,254,11,0,2,110,97,109,101,0,5,100,0,0,0,12,244 ]
    },
    {
        desc: `test: 写入 变量名: 'isOk', 变量值: true`,
        data: ['isOk', true],
        expect: [ 243, 251, 8, 0, 0x02, 105, 115, 79, 107, 0, 11, 1, 164, 244 ]
    },
    {
        desc: `test: 写入 变量名: 'pi', 变量值: 3.14`,
        data: ['pi', 3.14],
        expect: [ 243, 252, 9, 0, 0x02, 112, 105, 0, 9, 194, 245, 72, 64, 35, 244 ]
    },
    {
        desc: `test: 写入 变量名: 'hello', 变量值: 'world'`,
        data: ['hello', 'world'],
        expect: [ 243, 0, 13, 0, 0x02, 104, 101, 108, 108, 111, 0, 12, 119, 111, 114, 108, 100, 74, 244 ]
    }
]

describe('测试组装通信变量协议: 键值对 => 协议', function() {
    for(let val of keyAndValues) {
        it(val.desc, ()=>{
            let protocol = Encodebytes.commVarBytes(val.data[0], val.data[1]);
            console.log(protocol.join(','));
            expect(protocol).be.deep.equal(val.expect);
        })
    }
});

// 传感器协议
const sensors = [
    {
        desc: `action: 1, sensorID: 100`,
        data: {
            action: 1,
            sensorID: 100
        },
        expect: [ 243,247,4,0, 4, 1, 100,0, 105,244 ]
    },
    {
        desc: `action: 2, sensorID: 100`,
        data: {
            action: 2,
            sensorID: 100
        },
        expect: [ 243,247,4,0, 4, 2, 100,0, 106,244 ]
    },
    {
        desc: `action: 2, sensorID: 300`,
        data: {
            action: 2,
            sensorID: 300
        },
        expect: [ 243,247,4,0, 4, 2, 44,1, 51,244 ]
    }
]
describe('测试组装传感器协议: actionID + sensorID => 协议', function() {
    for(let val of sensors) {
        it(val.desc, ()=>{
            let protocol = Encodebytes.sensorBytes(val.data.action, val.data.sensorID);
            console.log(protocol.join(','));
            expect(protocol).be.deep.equal(val.expect);
        })
    }
});

// 广播协议
const broadcast = [
    {
        desc: `msg: 'a'`,
        data: 'a',
        expect: [ 243,246,3,0, 3,97,0, 100,244 ]
    },
    {
        desc: `msg: 'na_me'`,
        data: 'na_me',
        expect: [ 243,250,7,0,3,110,97,95,109,101,0,3,244 ]
    },
    {
        desc: `msg: '100'`,
        data: '100',
        expect: [ 243,248,5,0,3,49,48,48,0,148,244 ]
    }
]
describe('测试组装广播协议: 广播消息 => 协议', function() {
    for(let val of broadcast) {
        it(val.desc, ()=>{
            let protocol = Encodebytes.broadcastBytes(val.data);
            console.log(protocol.join(','));
            expect(protocol).be.deep.equal(val.expect);
        })
    }
});

