/**
 * Codey 协议模拟测试数据
 */

const nullResultCase = [
    {
        desc: '无头无尾，无效数据',
        data: [
            [8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118]
        ],
        expect: null
    }, {
        desc: '有头无尾数据', //等待下一次数据
        data: [
            [0xf3, 251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118]
        ],
        expect: undefined
    }, {
        desc: '冒牌数据: 包 length 比标记的短',
        data: [
            [0xf3, 252, 9, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: undefined
    }, {
        desc: '冒牌数据: 包 length 比标记的长',
        data: [
            [0xf3, 250, 7, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: null
    }, {
        desc: '冒牌数据: checksum 校验失败',
        data: [
            [0xf3, 251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 127, 0xf4]
        ],
        expect: null
    }, {
        desc: '有头无尾数据，且已超长',
        data: [
            [0xf3, 251, 8, 0, 0x02],
            [97, 0, 9, 0, 0, 200],
            [66, 118, 252]
        ],
        expect: null
    }, {
        desc: '假数据头 + 完整数据',
        data: [
            [5, 1, 24, 0xf3, 254, 11, 0, 0x02, 110],
            [97, 109, 0xf3, 251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: null
    }
]

const oneResultCase = [
    {
        desc: '完美数据1',
        data: [
            [0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: {
            pack: 1
        }
    }, {
        desc: '完整数据 + 头部之前有数据混入',
        data: [
            [5, 1, 24, 0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: {
            pack: 1
        }
    }, {
        desc: '完整数据 + 尾部之后有数据混入',
        data: [
            [0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4, 5, 1, 24]
        ],
        expect: {
            pack: 1
        }
    }, {
        desc: '完整数据出现了 2 个头部',
        data: [
            [0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 0xf3, 66, 161, 0xf4]
        ],
        expect: {
            pack: 1
        }
    }, {
        desc: '完整数据出现了 2 个尾部',
        data: [
            [0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 0xf4, 66, 162, 0xf4]
        ],
        expect: {
            pack: 1
        }
    }, {
        desc: '完整数据出现了 2 个头部 + 头部之前有数据混入',
        data: [
            [5, 1, 0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 0xf3, 66, 161, 0xf4]
        ],
        expect: {
            pack: 1
        }
    }, {
        desc: '完整数据出现了 2 个尾部 + 尾部之后有数据混入',
        data: [
            [0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 0xf4, 66, 162, 0xf4, 5, 1]
        ],
        expect: {
            pack: 1
        }
    }
]

const twoResultComplexCase = [
    {
        desc: '2条正确的数据被分成了 2 个包',
        data: [
            [5, 1, 24, 0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0],
            [200, 66, 118, 0xf4, 0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: {
            pack: 2
        }
    }, {
        desc: '2条正确的数据被分成了 3 个包',
        data: [
            [5, 1, 24, 0xf3,251, 8, 0, 0x02, 97, 0, 9, 0, 0],
            [200, 66, 118, 0xf4, 0xf3,251, 8, 0, 0x02],
            [97, 0, 9, 0, 0, 200, 66, 118, 0xf4]
        ],
        expect: {
            pack: 2
        }
    }, {
        desc: '2条正确的数据被分成了 4 个包',
        data: [
            [5, 1, 24, 0xf3, 251, 8, 0, 0x02, 97, 0],
            [9, 0, 0, 200, 66, 118, 0xf4, 0xf3],
            [251, 8, 0, 0x02, 97, 0, 9, 0],
            [0, 200, 66, 118, 0xf4]
        ],
        expect: {
            pack: 2
        }
    }
]



// 数据来源于真实测试
const truelyResultComplexCase = [
{
    desc: '一条数据包含了3个包，该数据能解析出 3 个包',
    data: [
        [0xf3, 251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4, 0xf3, 251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4,
        0xf3, 251, 8, 0, 0x02, 97, 0, 9, 0, 0, 200, 66, 118, 0xf4, 51, 0, 0]
    ],
    expect: {
        pack: 3
    },
},
{
    desc: '1条正确的数据被分成了 4 个包',
    data: [[243,251,8,0,5,1,48,174,164,32,171,184,11,244,
           243,251,8,0,5,2,48,174,164,32,171,186,14,244,
           243,1,14,0,6,50,51,46,48,49,46,48,48,52,46,48,48,50,124,244,
           243,252,9,0,9,0,4,5,85,65,82,84,48,126,244
    ]],
    expect: {
        pack: 4
    }
}]


/********************************************/
/*             具体协议的模拟数据              */
/********************************************/
//传感器读值协议：解析协议结果
const sensorProtocol2Result = [{
    desc: `should parse received protocol to be {id: 300, action: 2, value: null}`,
    data: [243,251,4,0,   4, 2,  44,1,   51,244],
    expect: {
        id: 300,
        action: 2
    }
},
{
    desc: `should parse received protocol to be {id: 300, action: 3, value: 0}`,
    data: [243,251,8,0,   4, 3,  44,1,  0,0,0,0,   52,244],
    expect: {
        id: 300,
        action: 3,
        value: 0
    }
},
{
    desc: `should parse received protocol to be {id: 300, action: 3, value: 0}`,
    data: [243,251,8,0,   4, 3,  44,1,  244,0,243,0,   27,244],
    expect: {
        id: 300,
        action: 3,
        value: 0
    }
},
{
    desc: `should parse received protocol to be {id: 300, action: 4, value: null}`,
    data: [243,251,4,0,   4, 4,  44,1, 53,244],
    expect: {
        id: 300,
        action: 4
    }
}];

//通信变量协议：解析协议结果
const commVarProtocol2Result = [{
    desc: `should parse received protocol to be {name: 'pi', value: 3.14 }`,
    data: [243,252, 9, 0, 0x02, 112, 105, 0, 9, 194, 245, 72, 64, 35, 244],
    expect: {
        name: 'pi',
        value: 3.14
    }
}, {
    desc: `should parse received protocol to be {name: 'name', value: 100 }`,
    data: [243,254, 11, 0, 0x02, 110, 97, 109, 101, 0, 9, 0, 0, 200, 66, 182, 244],
    expect: {
        name: 'name',
        value: 100
    }
}, {
    desc: `should parse received protocol to be {name: 'isOk', value: true }`,
    data: [243,251, 8, 0, 0x02, 105, 115, 79, 107, 0, 11, 1, 164, 244],
    expect: {
        name: 'isOk',
        value: true
    }
}, {
    desc: `should parse received protocol to be {name: 'hello', value: 'world' }`,
    data: [243,0, 13, 0, 0x02, 104, 101, 108, 108, 111, 0, 12, 119, 111, 114, 108, 100, 74, 244],
    expect: {
        name: 'hello',
        value: 'world'
    }
}, { // 来自下位机数据
    desc: `should parse received protocol to be {name: 'a', value: 'abc' }`,
    data: [243,250,7,0,2,97,0,12,97,98,99,149,244],
    expect: {
        name: 'a',
        value: 'abc'
    }
}];

//广播协议：解析协议结果
const broadcastProtocol2Result = [
 {
    desc: `should parse received protocol to be 'na_me'`,
    data: [ 243,250,7,0,3,110,97,95,109,101,0,3,244 ],
    expect: 'na_me'
}]

//查询 MAC 地址的协议：解析协议结果
const macAddrProtocol2Result = [
 {
    desc: `should parse received protocol to be 'ble mac address object'`,
    data: [243, 251, 8, 0, 5, 2, 48, 174, 164, 39, 222, 108, 250, 244],
    expect: {
        mac: '30 AE A4 27 DE 6C',
        type: 'ble'
    }
}]

//查询版本号协议：解析协议结果
const versionProtocol2Result = [
 {
    desc: `should parse received protocol to be 'version object'`,
    data: [243, 8, 21, 0, 6, 50, 51, 46, 48, 49, 46, 84, 48, 55, 46, 86, 49, 46, 48, 97, 108, 112, 104, 97, 51, 47, 244],
    expect: {
        version: "23.01.T07.V1.0alpha3",
        name: "codey"
    }
}]

//ready协议：解析协议结果
const readyProtocol2Result = [
 {
    desc: `should parse received protocol to be 'ready object'`,
    data: [243, 245, 2, 0, 8, 128, 136, 244],
    expect: {
        type: 0x80 // ready
    }
}]

//查询通信信道协议：解析协议结果
const commChannelProtocol2Result = [
 {
    desc: `should parse received protocol to be 'comm channel info object'`,
    data: [243, 252, 9, 0, 9, 0, 4, 5, 85, 65, 82, 84, 48, 126, 244],
    expect: {
        channel: "UART0"
    }
}]

export {
    nullResultCase,
    oneResultCase,
    twoResultComplexCase,
    truelyResultComplexCase,
    sensorProtocol2Result,
    commVarProtocol2Result,
    broadcastProtocol2Result,
    macAddrProtocol2Result,
    versionProtocol2Result,
    readyProtocol2Result,
    commChannelProtocol2Result
}