import {
    arrayFromArrayBuffer
} from '../src/util';
import {
    nullResultCase,
    oneResultCase,
    twoResultComplexCase,
    truelyResultComplexCase,
    sensorProtocol2Result,
    commVarProtocol2Result,
    broadcastProtocol2Result,
    macAddrProtocol2Result,
    versionProtocol2Result,
    readyProtocol2Result,
    commChannelProtocol2Result
} from './mock-received-protocol';

import Decodebytes from '../src/decodebytes';
import Parse from '../src/parse';
import chai from 'chai';
const expect = chai.expect;

//协议解包测试
describe('协议解包测试: 接收协议包，并对协议进行解析', function() {
    for (let case_ of nullResultCase) {
        it(case_.desc, function() {
            Decodebytes.responseCache = [];
            let data = case_.data;
            let result;
            for (let data of case_.data) {
                result = Decodebytes.splitFrame(arrayFromArrayBuffer(data));
            }
            expect(result).to.equal(case_.expect);
        });
    }

    let testCases = [...oneResultCase, ...twoResultComplexCase, ...truelyResultComplexCase];
    for (let case_ of testCases) {
        it(case_.desc, function() {
            Decodebytes.responseCache = [];
            let data = case_.data;
            let results = [];
            for (let data of case_.data) {
                let result = Decodebytes.splitFrame(arrayFromArrayBuffer(data)); //[[]]
                if(result) {
                    results = results.concat(result);
                }
            }
            expect(results.length).to.equal(case_.expect.pack);
        });
    }
});



//接收通信变量协议自测用例
describe('通信变量协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of commVarProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getNameAndValue(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});

//接收广播协议协议自测用例
describe('广播协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of broadcastProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getBroadcastMsg(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});

//传感器读值协议协议自测用例
describe('传感器读值协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of sensorProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getSensorVal(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});

//mac 地址查询协议自测用例
describe('mac 地址查询协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of macAddrProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getMacAddr(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});

//版本 version 协议测试
describe('版本 version 协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of versionProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getVersion(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});

//ready 协议测试
describe('ready 协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of readyProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getReadyResult(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});

//查询通信信道协议测试
describe('查询通信信道协议测试: 对接收到的协议，解析出结果', function() {
    for (let case_ of commChannelProtocol2Result) {
        it(case_.desc, function() {
            let data = case_.data;
            let result = Parse.getCommChannel(data);
            expect(result).be.deep.equal(case_.expect);
        });
    }
});